/**********************************************************************
 * File Name: SimEnvironment.java
 *
 * Copyright (C) 2000, US Air Force Research Laboratory,
 * Rome Research Site, Rome NY 13424, USA.
 *
 * Use of this software implies acceptance of the terms described in
 * accompanying license.txt file.
 *
 *
 * Authors: James Lawton, Jennifer Dzek, John Crowter,
 *          Bob Paragi, and Joe Beasock
 *
 * Creation Date: July 1999
 *
 * Description:
 *
 * Modification History:
 *
 *  - June 00, Bryan Horling, UMass (BCH): Changes to make drawing more 
 *  efficient by only redrawing when needed.
 *
 **********************************************************************/

import java.io.*;
import java.util.*;
import java.awt.*;

public class SimEnvironment {

    public int Debug;
    public int width;
    public int height;
    public boolean auto_input;
    public ArrayList mobile_objects;
    public ArrayList sensors;
    public ArrayList sim_objects;
    public String name;
    public boolean running;
    public boolean quitCalled;
    public SimClock clock;
    public RadsimCanvas Display = null;
    public RadsimStatus Status = null;
    public int nextAgentID;
    public CommChannel commChannels[];
    public float time_ratio = 1;
    public boolean redraw = true, fullredraw = true; //BCH


    public SimEnvironment() {
	width =100;
	height = 100;
	mobile_objects = new ArrayList();
	sensors = new ArrayList();
	sim_objects = new ArrayList();
	name = "RadSim Environment";
	auto_input = false;
        running = false;
        quitCalled = false;
	commChannels = new CommChannel[8];
	for (int i = 0; i < 8; i++) {
	    commChannels[i] = new CommChannel("Channel-" + i, this);
	    sim_objects.add(commChannels[i]);
	}
	nextAgentID = 0;
    }

    public SimEnvironment(int w, int h) {
	Debug = 0;
	width = w;
	height = h;
	mobile_objects = new ArrayList();
	sensors = new ArrayList();
	sim_objects = new ArrayList();
	name = "RadSim Environment";
	auto_input = false;
        running = false;
        quitCalled = false;
	commChannels = new CommChannel[8];
	for (int i = 0; i < 8; i++) {
	    commChannels[i] = new CommChannel("Channel-" + i, this);
	    sim_objects.add(commChannels[i]);
	}
	nextAgentID = 0;
    }

    public void add_sensor(Sensor obj) {
	sensors.add(obj);
	sim_objects.add(obj);
	obj.env = this;
	requestFullRedraw(); //BCH
    }

    public void add_mobile_object(MobileObject obj) {
	mobile_objects.add(obj);
	sim_objects.add(obj);
	obj.env = this;
	requestRedraw(); //BCH
    }

    public int scale_x_coord(double x, double screen_width) {
	double xratio = screen_width / width;
	return ( (int) (x * xratio) );
    }

    public int scale_y_coord(double y, double screen_height) {
	double yratio =  screen_height / height;
	return ( (int) (screen_height - (y * yratio)) );
    }

    public void find_object_at(double x, double y, double screen_height, double screen_width) {
	double obj_x = (width  / screen_width)  * x;
	double obj_y = (height / screen_height) * (screen_height - y);
	double near = 2.0;
  	Sensor obj = null;
	boolean found = false;
	int i;
	String menuItems[] = null;

	for (ListIterator objs = sensors.listIterator(); (! found) && objs.hasNext(); ) {
	    obj = (Sensor) objs.next();
	    if ( Math.abs(obj_x - obj.xS) < near &&
		 Math.abs(obj_y - obj.yS) < near )
		found = true;
	}
	if (found) {
	    if (Debug > 1)
               System.out.println("Mouse click at near " + obj.name);
	    if (Display != null && Display.infoMenu != null) {
		PopupMenu info = Display.infoMenu;
		info.removeAll();
		menuItems = obj.getSensorInfo();
		for (i = 0; i < menuItems.length; i++)
			 info.add(menuItems[i]);
		info.show(Display, (int) x, (int) y);
	    }
	}
    }

    public void update_display() {
	//Display.repaint();
	if (Display != null && Status != null) {
      if (needsRedraw()) {
          Display.update(Display.getGraphics());
      } else {
          // Do nothing
      }
      Status.update(Status.getGraphics());
	    //try {Thread.sleep(500);} catch (Exception e){};
	}
    }


    public synchronized int getNextAgentID () {
	return(nextAgentID++);
    }

    // Accessor routines to tell the environment when to redraw (BCH)
    public synchronized boolean needsRedraw() {
        return (redraw || fullredraw);
    }
    public synchronized void requestRedraw() {
        redraw = true;
    }
    public synchronized boolean needsFullRedraw() {
        return fullredraw;
    }
    public synchronized void requestFullRedraw() {
        fullredraw = true;
    }
    public synchronized void handleRedraw() {
        redraw = false;
        fullredraw = false;
    }

}
